function mephist(gnum, anode, cathode, current, varargin)
% This function plots the MEP IEMG histograms for a given configuration
% averaged across animals and across the weeks.
% USAGE
% MEPHIST(GNUM, ANODE, CATHODE, CURRENT)
% gnum: group number as an integer
% anode: the stimulating anode as a string of format 'C#' or 'Rf'
% cathode: the stimulating cathode as a string of format 'C#' or 'Rf'
% current: the stimulating current used for the experimemt as an integer
% SD flag: a boolean flag used to toggle the plotting of SD and can be used in
% the following method i.e. mephist(3, 'C6', 'C8', 300, true) or
% mephist(3,'C6',C8', 300, false) or mephist(3,'C6',C8', 300)

close all;
if ~isempty(varargin) && varargin{1}
    sd_flag = true;
else
    sd_flag = false;
end

% Load common variables
config();
load('config.mat');
load('baseline_struct.mat', 'baseline_struct');

% Selected directories
sel_dirs = {'.\Data\B_animals_sMEP_files\Intact', ...
            '.\Data\B_animals_sMEP_files\W01', ...
            '.\Data\B_animals_sMEP_files\W06'};

% Specific group for analysis
gnum_str = num2str(gnum);
gname = ['Group' gnum_str];

pulse_thold = SLOPE;
nelec_ch = 5;

% Max number of meps for plotting
nmep_lim = 75;

% Specify the bin size across the 35ms window to calculate IEMG across MEPs
sbin = 0.005; % 5 ms

% fontsize for any labels and tick labels
bigfontsize = 16;
smlfontsize = 10;
% Specify the labelling for the bottom xticks 
xticklabels = { '-10:-5', '-5:0' , '0:5', '5:10', ...
                '10:15', '15:20', '20:25', '25:30'};

% files of interest
ndirs = length(sel_dirs);

MEP_time = (-(PRE3_MEP+1)/fs:1/fs:POST3_MEP/fs)*1000';

% Strings for plot titles
week_temp = split(sel_dirs,'\');
week_names = week_temp(:,:,end);
if gnum == 3
    plt_week = {week_names{1} week_names{2} ...
        [week_names{2} ' BR'] week_names{3}};
else
    plt_week = week_names;
end

foi_inds = length(plt_week);
foi_ind = 1;
ch_names = {'Deltoid', 'Bicep', 'Pronator', 'Flexor', 'Extensor'};

foi = cell(1,1);
grey = ones(3,1).*0.6;

% Loop through selected directories to get all the relevant files of interest
% which match the given configuration
for nsel = 1 : ndirs
    sel_dir = cell2mat(sel_dirs(nsel));
    addpath(genpath(sel_dir));

    g_dir = [sel_dir '\' gname '\'];

    files = dir([g_dir '\*.bin']);
    fnames = {files.name};
    fnames = cellfun(@(x) horzcat(g_dir, x), fnames, 'UniformOutput', false);
    fileinfo_arr = catFilenames(fnames');

    % Match the configuration within the selected directory/group
    anode_arr = catStructField(fileinfo_arr, 'Anode');
    cathode_arr = catStructField(fileinfo_arr, 'Cathode');
    current_arr = catStructField(fileinfo_arr, 'Current');
    current_arr = cell2mat(current_arr);

    arr_inds = 1 : length(anode_arr);

    anode_mask = strcmp(anode_arr, anode);
    cathode_mask = strcmp(cathode_arr, cathode);
    curr_mask = current_arr == current;

    inds = arr_inds(anode_mask & cathode_mask & curr_mask);
    temp = fnames(inds);
    br_mask = contains(temp, 'BR');
    br_inds = inds(br_mask);
    nr_inds = inds(~br_mask);
    foi{:,1} = fnames(nr_inds)';
    foi{:,2} = fnames(br_inds)';

    % AnNums{:,1} = getUniqueAnNum(foi, 1);
    % AnNums{:,2} = getUniqueAnNum(foi, 2);
    for c = 1 : size(foi,2)
        foi_len = length(foi{:,c});
        ffoi = foi{c};
        for n = 1 : foi_len
            chosenfile = ffoi{n};
            % Load emg data
            EMG_header = load_emg_header(chosenfile, 1);
            data_array = load_emg_data(chosenfile, ...
                EMG_header,EMG_header.data_position,1,inf,1);
            inds = 1 : size(data_array,1);
            emg_data = data_array(:,1:nelec_ch);
            stim_pulse = data_array(:,6);

            % Filter and normalize against the given configuration baseline
            emg_filt = bpFilter(emg_data, 2, 30, 1e3, nq);
            [~, emg_tkeo] = energyop(emg_filt, false);
            emg_tkeo = reshape(emg_tkeo,size(emg_filt));
            emg_env = abs(lpFilterEMG(emg_tkeo, 2, 50, nq));
            if contains(chosenfile, 'Intact')
                emg_ac = emg_filt;
            else
                heading = chosenfile(1:end-4);
                temp = find(heading(2:end) == 'W') + 1;
                an = heading(temp+4:temp+6);
                emg_norm = normalizeEMGData(emg_filt, ...
                    baseline_struct, an, anode, cathode, current);
                emg_ac = emg_norm;
            end

            % Find threshold
            thold = thresholding(emg_env, THR_MUL);
            active_mask = emg_env > thold;
            new_stim = map_stimpulse(active_mask, stim_pulse);
            for ch = 1 : nelec_ch - 1
                if ch == 1
                    final_stim = and(round(new_stim(:,ch)), ...
                        round(new_stim(:,c+1)));
                else
                    final_stim = and(final_stim, ...
                        round(new_stim(:,ch+1)) );
                end
            end

            % Set pulse offset and get mep traces
            pulse_offset = [final_stim(2:end); NaN];
            risingEdge = inds(final_stim < pulse_thold & ...
                pulse_offset > pulse_thold);

            counter = risingEdge';
            meps = meptrace(emg_ac, counter);
            out_MEPs = meps.out_MEPs;

            % Limit number of traces to nmep_lim variable
            if size(out_MEPs,2) > nmep_lim
                plot_MEPs = out_MEPs(:, 1:nmep_lim, :);
            else
                plot_MEPs = out_MEPs;
            end

            % meps, nmeps, chs
            [nsamples, nMEPs, nchannels] = size(plot_MEPs);

            % reshape the 401 x nmeps x 5 to be in bins of 5ms so
            % reshape to  0.005*fs x 400/(0.005*fs) x nmeps x nchs
            meps_ = reshape(plot_MEPs, [sbin*fs, nsamples/(sbin*fs), ...
                nMEPs, nchannels]);
            meps_iemg = trapz(MEP_time(1:sbin*fs), abs(meps_), 1);
            meps_iemg = squeeze(meps_iemg);

            % iemgbin x nch x an
            mu_iemg(:, :, n) = squeeze(mean(meps_iemg, 2));
            sd_iemg(:, :, n) = squeeze(std(meps_iemg, [], 2)./sqrt(nMEPs));
            % Now to separate in to teatment and time point
        end
        % If we have our variables ready, lets do calculations
        if exist('mu_iemg', 'var')
            % Mean for each animal
            mu = squeeze(mean(mu_iemg, 3));
            % Standard deviation for each animal
            sd = squeeze(std(mu_iemg, [], 3));
            % Standard error for the number of detected MEPs
            sd_err = sd./sqrt(size(mu_iemg,3));
            if c == 1
                treat = 'ND';
            elseif c == 2
                treat = 'BR';
            end
            % Save to a data struct for later use
            mep_struct.(treat).(week_names{nsel}).mu = mu;
            mep_struct.(treat).(week_names{nsel}).sd = sd;
            mep_struct.(treat).(week_names{nsel}).sd_err = sd_err;
            clear mu_iemg sd_iemg;
        end
        % Plot on 1 figure
        if ~isempty(foi{c})
            buff = foi_ind - 2;
            sp_off = 1;
            % Mean of means for each animal
            mu = squeeze(mu);
            sd = squeeze(sd_err);
            for ch = 1 : nelec_ch
                % subplot(nelec_ch, 1, ch);
                subplot(nelec_ch, length(plt_week), ...
                    sp_off + ch + buff);
                hold on;
                % Here is our bar plot of mean IEMG for the respective bin size
                bar(mu(:,ch), 'facecolor', grey);
                % Plot our error bars here for standard error
                errorbar(mu(:,ch), sd(:,ch), 'k.');
                hold off;
                % Some formatting to make the titles and labels pretty
                if ch == 1
                    % Plot titles only if we are at the top of the graph
                    title_str = [plt_week{foi_ind} ...
                        ' (n = ' num2str(foi_len) ')'];
                    title(title_str, 'interpreter', 'none', ...
                    'fontsize',bigfontsize);
                end
                if ch < nelec_ch
                    % Ignore any xtick labels but keep the space between 1 and
                    % 8 if we are not at the bottom of the graph
                    set(gca, 'xtick', 1 : 8, 'xticklabels', {[]});
                else
                    % Lets print our xticklabels if we're at the bottom row
                    set(gca, 'xtick', 1 : 8, 'xticklabels', ...
                        xticklabels, 'fontsize', smlfontsize);
                    % Rotate to make sure everything fits in by 90deg
                    xtickangle(90);
                    % If we're at the bottom of the graph and in the middle of
                    % the columns, lets print our xlabel
                    if nsel == 2 && c == 1
                        xlabel('MEP 5ms bin', 'fontsize', bigfontsize);
                    end
                end
                if nsel == 1
                    % Plot ylabels only if we are in the middle of the graph
                    if ch == 3
                        ylabel({'\mu_{IEMG} \pm \sigma/\surd{n}', ...
                            ch_names{ch}}, 'fontsize', bigfontsize);
                    else
                        ylabel(ch_names{ch}, 'fontsize', bigfontsize);
                    end
                end

                if foi_inds == 3
                    sp_off = sp_off + 2;
                elseif foi_inds == 4
                    sp_off = sp_off + 3;
                end
                % set(gca,'fontsize',fontsize);
            end
            foi_ind = foi_ind + 1;
        end
    end
end


end

function f_arr = catFilenames(files)
% Concatenate all the files and organise by configurable experiment variables
    f_arr = struct();
    for f = 1 : length(files)
        fileinfo = getFileInfo(files{f});
        Week = fileinfo.Week;
        AnNum = fileinfo.AnNum;
        Anode = fileinfo.Anode;
        Cathode = fileinfo.Cathode;
        Current = fileinfo.Current;

        f_arr(f).Week = Week;
        f_arr(f).AnNum = AnNum;
        f_arr(f).Anode = Anode;
        f_arr(f).Cathode = Cathode;
        f_arr(f).Current = Current;
    end
end

function AnNums = getUniqueAnNum(foi, n)
% This function extracts the unique animal ID based on the given file of
% interest cell array and which column of that array you're interested in, note
% that the first column is ND and second is BR
    files = foi{:,n};
    flen = length(files);
    for f = 1 : flen
        fstruct = getFileInfo(files{f});
        AnNums{f} = fstruct.AnNum;
    end
end
