function [iemg, pk2pk] = mepresponse(filt, stim, fs, varargin)
%
%   Author: Raymond Chia
%   Date:   17/Oct/2018
%   Summary:
%   MEPRESPONSE takes an array of AC EMG data and synchronised stim pulses to
%   determine the motor evoked potential response. Output information is the
%   integrated EMG and peak-to-peak values in the middle and late response
%   windows. 
%   
%   USAGE
%   [iemg , pk2pk] = mepresponse(filt, stim, fs)
%   
%    
    
    if (nargin > 3)
        ch = varargin{1};
    else
        ch = [];
    end
    
    % Load the configuration variables
    [timing_cfg , thold_cfg, plt_cfg] = mep_config();
    [nrows, ncols] = size(filt);

    minStimPkDist   = thold_cfg.minStimPkDist;
    minStimPkHeight = thold_cfg.minStimPkHeight;

    plot_flag = plt_cfg.plot_flag;
    save_flag = plt_cfg.save_flag;

    % Find the peaks for the stim pulses to indicate when each stim pulse
    % occurs
    [pks,ix] = findpeaks(stim, 'MinPeakDistance',minStimPkDist,...
        'MinPeakHeight', minStimPkHeight);
    % If there are no stim pulses detected, return all zeros
    if isempty(pks)
        iemg = struct();
        mr_iemg = [0 0];
        lr_iemg = [0 0];
        iemg.mr = mr_iemg;
        iemg.lr = lr_iemg;

        pk2pk = struct();
        mr_pk = [0 0];
        mr_means = [0 0];
        mr_num_pks = [0 0];
        pk2pk.mr = mr_pk;
        pk2pk.mr_means = mr_means;
        pk2pk.mr_num_pks = mr_num_pks;

        lr_pk = [0 0];
        lr_means = [0 0];
        lr_num_pks = [0 0];
        pk2pk.lr = lr_pk;
        pk2pk.lr_means = lr_means;
        pk2pk.lr_num_pks = lr_num_pks;
        return;
    end

    % Initialise variables
    mr_start = timing_cfg.mr_start*fs;
    mr_end   = timing_cfg.mr_end*fs;
    lr_start = timing_cfg.lr_start*fs;
    lr_end   = timing_cfg.lr_end*fs;

    minPkDist   = thold_cfg.minPkDist;
    minPkProm   = thold_cfg.minPkProm;
    maxPkWidth  = thold_cfg.maxPkWidth;
    minPkHeight = thold_cfg.minPkHeight;
    diff2_thold = thold_cfg.diff2_thold;

    n = length(ix);

    % Define window to be the index of each stim peak to the lr_end variable
    win = ix + lr_end;

    if win(end) > nrows
        ix = ix(1 : end - 1);
        pks = pks(1 : end - 1);
        win = ix + lr_end;
        n = length(ix);
    end
    mr_iemg = zeros(n , ncols);
    lr_iemg = zeros(n , ncols);
    mr_pk   = cell(n, ncols);
    lr_pk   = cell(n, ncols);
    mr_num_pks = zeros(n , ncols);
    lr_num_pks = zeros(n , ncols);
    mr_means = zeros(n , ncols);
    lr_means = zeros(n , ncols);

    scale = timing_cfg.interp_scale;
    pk_window = timing_cfg.window*fs*scale;

    % For each stim pulse, loop
    for i = 1 : n
        % Working window
        inds = ix(i) : win(i) - 1;
        t = inds/fs;
        % Create out interpolation 'x' axis
        t1 = linspace(t(1),t(end),length(t)*scale);
        inds1 = 1 : length(t1);
        % Define out middle and late response windows
        mr_inds = ix(i) + mr_start : ix(i) + mr_end - 1;
        lr_inds = ix(i) + lr_start : ix(i) + lr_end - 1;
        lr_inds1 = inds1(1) + lr_start*scale : inds1(1) + lr_end*scale - 1;
        mr_inds1 = inds1(1) + mr_start*scale : inds1(1) + mr_end*scale - 1;
        % For each of the columns in our 'filt' array
        for j = 1 : ncols
            % Initialise and interpolate
            resp = filt(inds,j);
            resp = interp1(t, resp, t1, 'spline');
            resp = resp';
            resp_norm = normalize(resp);

            % Find our peaks based on the threshold value
            [lmax, pval_max] = islocalmax(resp_norm, 'MinSeparation', minPkDist, ...
                'SamplePoints', t1, 'FlatSelection', 'first', ...
                'MinProminence', minPkProm);
            pix = inds1(lmax);
            % Find our troughs based on the threshold value
            [lmin, nval_min] = islocalmax(-resp_norm, 'MinSeparation', minPkDist, ...
                'SamplePoints', t1, 'FlatSelection', 'first', ...
                'MinProminence', minPkProm);
            nix = inds1(lmin);

            % Differentiate the data to check against the second differential
            % threshold 'diff2_thold'
            resp_diff = differentiate(resp_norm,   fs*scale, 1);
            resp_diff2 = differentiate(resp_diff,  fs*scale, 1);
            resp_diff3 = differentiate(resp_diff2, fs*scale, 1);
            df_chk = abs(resp_diff2) > diff2_thold;
            df_chk = [false(2,1) ; df_chk];

            % Create a mask for the troughs and valleys that meet the second
            % differential threshold
            lmax_mask = logical(sum(inds1' == pix, 2)) & df_chk;
            lmin_mask = logical(sum(inds1' == nix, 2)) & df_chk;
            mask = lmax_mask | lmin_mask;

            % Calculate iemg
            mr_iemg(i , j) = calculateIEMG(filt(mr_inds, j), mr_inds ./ fs);
            lr_iemg(i , j) = calculateIEMG(filt(lr_inds, j), lr_inds ./ fs);

            % Initialise the peaks indexes
            pk_inds = inds1(mask);
            vec = 1 : length(pk_inds);
            % associate each one to mr or lr sects
            temp = logical(sum(mr_inds1' == pk_inds));
            mr_pk_inds = pk_inds(vec(temp));
            mr_pk_vals = resp(mr_pk_inds);
            temp = logical(sum(lr_inds1' == pk_inds));
            lr_pk_inds = pk_inds(vec(temp));
            lr_pk_vals = resp(lr_pk_inds);

            if plot_flag
                if isempty(ch)
                    plotFigures(resp, resp_diff, resp_diff2, resp_diff3, fs,...
                        inds1, mr_inds1, lr_inds1, mask, j);
                else
                    plotFigures(resp, resp_diff, resp_diff2, resp_diff3, fs,...
                        inds1, mr_inds1, lr_inds1, mask, ch);
                end
                % UNCOMMENT HERE TO PAUSE 
                % fprintf('paused...\n');
                % pause();
            end

            % Calculate peak to peak values
            mr_pk{i , j} = peak2peak(lmax_mask, lmin_mask, resp, ...
                mr_inds1, pk_window);
            mr_pk_vals = mr_pk{i,j};
            mr_num_pks(i,j) = length(mr_pk_vals);
            temp = isnan(mr_pk_vals);
            mr_means(i,j) = mean(mr_pk_vals(~temp));
            lr_pk{i , j} = peak2peak(lmax_mask, lmin_mask, resp, ...
                lr_inds1, pk_window);
            lr_pk_vals = lr_pk{i,j};
            lr_num_pks(i,j) = length(lr_pk_vals);
            temp = isnan(lr_pk_vals);
            lr_means(i,j) = mean(lr_pk_vals(~temp));
        end
    end
    iemg = struct();
    iemg.mr = mr_iemg;
    iemg.lr = lr_iemg;

    pk2pk = struct();
    pk2pk.mr = mr_pk;
    pk2pk.mr_means = mr_means;
    pk2pk.mr_num_pks = mr_num_pks;
    pk2pk.lr = lr_pk;
    pk2pk.lr_means = lr_means;
    pk2pk.lr_num_pks = lr_num_pks;

end

function plotFigures(filt, filt_diff, filt_diff2, filt_diff3, fs, inds,...
    mr_inds, lr_inds, mask, ch_num)
    % This function plots the peaks, troughs, first, second and third signal
    % differentials

    [timing_cfg , ~ , plt_cfg] = mep_config();

    mu = mean(filt);
    sd = std(filt);
    if ~mod(ch_num,2)
        fig1 = 5;
        fig2 = 6;
    else
        fig1 = 3;
        fig2 = 4;
    end

    h1 = figure(fig1);
    scale = timing_cfg.interp_scale;
    t = inds.*1e3./(scale*fs);
    plot(t, filt, t(mr_inds), filt(mr_inds), t(lr_inds), ...
        filt(lr_inds) ); hold on;
    plot(t(inds(mask)), filt(mask), 'k*');
    plot([t(1) t(end)],[mu mu],'b');
    plot([t(1) t(end)],[mu+sd mu+sd], 'r');
    plot([t(1) t(end)],[mu-sd mu-sd], 'r');
    xlabel('Time (ms)');
    ylabel('EMG (mV)');
    grid on;
    hold off;

    % h2 = figure(fig2);
    % ax(1) = subplot(2,2,1); plot(filt); title('disp');
    % ax(2) = subplot(2,2,2); plot(filt_diff); title('vel');
    % ax(3) = subplot(2,2,3); plot(filt_diff2); title('accel');
    % ax(4) = subplot(2,2,4); plot(filt_diff3); title('jerk');
    % linkaxes(ax,'x');

    if plt_cfg.save_flag
        ext = '*.png';
        fig_dir = plt_cfg.fig_dir;
        file_list = dir([fig_dir ext]);
        temp = contains({file_list.name}, ['CH' int2str(ch_num)]);
        num_files = sum(temp);
        if ~isempty(ch_num)
            s1 = sprintf('%s%sCH%d_%d.png', fig_dir, plt_cfg.file_name, ch_num, num_files+1);
        else
            s1 = sprintf('%s%sCH%d_%d.png', fig_dir, plt_cfg.file_name, 0, num_files+1);
        end
        saveas(h1, s1);
    end
end

function pk2pk = peak2peak(lmax_mask, lmin_mask, y, sect, buff)
    % This function cycles through each peak and looks forwards for any
    % valleys, if there are none in the defined window, it looks backwards.
    % Window is defined with the argument, 'buff'. 
    num_pks = 0;

    % Define the peak indexes
    pk_mask = lmax_mask(sect);
    pk_inds = sect(pk_mask);

    % Define the peaks
    y_sect = y(sect);
    pks = y_sect(pk_mask);
    pk2pk = nan;

    % Loop through each peak
    for i = 1 : length(pk_inds)
        % Select first peak
        pk = pks(i);
        pk_ind = pk_inds(i);

        % Look at the forward indexes
        fwd_inds = pk_ind : pk_ind + buff;
        if fwd_inds(end) > sect(end)
            fwd_inds = pk_ind : sect(end);
        end
        % Check for troughs
        temp = lmin_mask(fwd_inds);
        temp = fwd_inds(temp);
        % If there are none, look behind
        if isempty(temp)
            bkw_inds = pk_ind - buff : pk_ind;
            if bkw_inds(1) < sect(1)
                bkw_inds = sect(1) : pk_ind;
            end
            % Check for troughs behind
            temp = lmin_mask(bkw_inds);
            temp = bkw_inds(temp);
            temp = fliplr(temp);
            % If there are none, then there is no peak to peak value for this
            % peak, move on to the next one
            if isempty(temp)
                num_pks = num_pks + 1;
                pk2pk(num_pks) = nan;
                continue;
            end
        end

        % If there is a trough then select the first found trough, calculate
        % the peak to peak value, then set the mask index to be false so we do
        % not count it again.
        vly_ind = temp(1);
        vly = y(vly_ind);
        num_pks = num_pks + 1;
        pk2pk(num_pks) = pk - vly;

        lmin_mask(vly_ind) = false;
        pk_mean = mean(pk2pk);
    end
end

function width = peakwidth(filt, pks, pks_inds)
    % Fix later
    width = nan;
    % In this current peak, where is it and when does it intersect next with the
    % signal
    for i = 1 : length(pks_inds)
        pk_ind = pks_inds(i);
    end
end
