function sMEPConfidenceInterval(anode, cathode, current, varargin)
% This function plots the MEP for a given configuration across the weeks.
% USAGE
% SMEPCONFIDENCEINTERVAL(ANODE, CATHODE, CURRENT)
% anode: the stimulating anode as a string of format 'C#' or 'Rf'
% cathode: the stimulating cathode as a string of format 'C#' or 'Rf'
% current: the stimulating current used for the experimemt as an integer
% SD flag: a boolean flag used to toggle the plotting of SD.

close all;
if ~isempty(varargin) && varargin{1}
    sd_flag = true;
else
    sd_flag = false;
end

config();
load('config.mat');

mep_struct = parseDataToStruct(anode, cathode, current, sd_flag);

amp_struct = mep_struct.amp;
latency_struct = mep_struct.latency;
auc_struct = mep_struct.auc;

weeks = {'W00', 'W01'};
responses = {'early', 'late'};
treatments = {'ND', 'BR'};

% Let's get all the animals in to one array for a response according to weeks
% in rows
% [    | week0  week1  ]
% [ ------------------ ]
% [an0 | x(0,0) x(0,1) ]
% [an1 | x(1,0) x(1,1) ]

counter = 1;
for r = 1 : length(responses)
    response = responses{r};
    for w = 1 : length(weeks)
        week = weeks{w};
        fnames = fieldnames(amp_struct.(response).(week));
        treat_chk = cell2mat(strfind(fnames, 'BR'));
        if any(treat_chk)
            treatments = {'ND', 'BR'};
        else
            treatments = {'ND'};
        end
        for t = 1 : length(treatments)
            treatment = treatments{t};
            amp_temp = amp_struct.(response).(week).(treatment);
            lat_temp = latency_struct.(response).(week).(treatment);
            auc_temp = auc_struct.(response).(week).(treatment);

            animals = fieldnames(amp_temp);
            for a = 1 : length(animals)
                an = animals{a};
                if a == 1
                    amp_arr_temp = amp_temp.(an);
                    lat_arr_temp = lat_temp.(an);
                    auc_arr_temp = auc_temp.(an);
                else
                    amp_arr_temp = ...
                        vertcat(amp_arr_temp, amp_temp.(an));
                    lat_arr_temp = ...
                        vertcat(lat_arr_temp, lat_temp.(an));
                    auc_arr_temp = ...
                        vertcat(auc_arr_temp, auc_temp.(an));
                end
            end
            N = length(animals);

            amp_mu = mean(amp_arr_temp);
            lat_mu = mean(lat_arr_temp);
            auc_mu = mean(auc_arr_temp);

            fprintf('%s: %s : %s : %d\n', response, treatment, week, N);
            amp_sd = std(amp_arr_temp)/sqrt(N);
            lat_sd = std(lat_arr_temp)/sqrt(N);
            auc_sd = std(auc_arr_temp)/sqrt(N);
            results(counter).response = response;
            results(counter).treatment = treatment;
            results(counter).week = week;

            results(counter).N = N;

            results(counter).amp_data = amp_arr_temp;
            results(counter).latency_data = lat_arr_temp;
            results(counter).auc_data = auc_arr_temp;

            results(counter).amp_mu = amp_mu;
            results(counter).latency_mu = lat_mu;
            results(counter).auc_mu = auc_mu;

            results(counter).amp_sd = amp_sd;
            results(counter).latency_sd = lat_sd;
            results(counter).auc_sd = auc_sd;
            counter = counter + 1;
        end
    end
end

N_max = max([results(:).N]);

for t = 1 : length(results)
    amp_data(:, :, t) = bufferData(results(t).amp_data, N_max);
    lat_data(:, :, t) = bufferData(results(t).latency_data, N_max);
    auc_data(:, :, t) = bufferData(results(t).auc_data, N_max);
end

period = [1 : 3; 4 : 6];
group = {'W00-ND' , 'W01-ND' , 'W01-BR'};
for s = 1 : 2
    per = period(s , :);
    for ch = 1 : size(amp_data,2)
        [amp_p(s,ch), ~, stats(s,ch)] = anova1(squeeze(amp_data(:,ch,per)),...
            group, 'off');
        [lat_p(s,ch), ~, stats(s,ch)] = anova1(squeeze(lat_data(:,ch,per)),...
            group, 'off');
        [auc_p(s,ch), ~, stats(s,ch)] = anova1(squeeze(auc_data(:,ch,per)),...
            group, 'off');
    end

    %{
    figure(1);
    mu_temp = [results(per).amp_mu];
    sd_temp = [results(per).amp_sd];
    yMean = reshape(mu_temp, length(per), length(mu_temp)/length(per));
    ySEM = reshape(sd_temp, length(per), length(sd_temp)/length(per));
    N_arr = [results(per).N];
    ax = plotCI(per, N_arr, ySEM, yMean, s);
    ylabel(ax(1,1),'EMG (mV)');
    if s == 2
        title(ax(1,1),'late - Ch1');
    else
        title(ax(1,1),{'Amplitude','early - Ch1'});
    end

    figure(2);
    mu_temp = [results(per).latency_mu];
    sd_temp = [results(per).latency_sd];
    yMean = reshape(mu_temp, length(per), length(mu_temp)/length(per));
    ySEM = reshape(sd_temp, length(per), length(sd_temp)/length(per));
    ax = plotCI(per, N_arr, ySEM, yMean, s);
    ylabel(ax(1,1),'Latency');
    if s == 2
        title(ax(1,1),'late - Ch1');
    else
        title(ax(1,1),{'Latency','early - Ch1'});
    end

    figure(3);
    mu_temp = [results(per).auc_mu];
    sd_temp = [results(per).auc_sd];
    yMean = reshape(mu_temp, length(per), length(mu_temp)/length(per));
    ySEM = reshape(sd_temp, length(per), length(sd_temp)/length(per));
    ax = plotCI(per, N_arr, ySEM, yMean, s);
    ylabel(ax(1,1), 'AUC (mVs)');
    if s == 2
        title(ax(1,1),'late - Ch1');
    else
        title(ax(1,1),{'AUC','early - Ch1'});
    end
    %}
end

amp_p
lat_p
auc_p

end

function ax = plotCI(period, N_arr, ySEM, yMean, s)
    if s == 1
        counter = 1;
    else
        counter = 6;
    end
    for p = 1 : length(period)
        N = N_arr(p);
        % Calculate 95% Probability Intervals Of t-Distribution
        CI95 = tinv([0.025 0.975], N-1)';                   
        for q = 1 : length(yMean)
            % Calculate 95% Confidence Intervals Of All Experiments 
            % At Each Value Of x
            yCI95 = bsxfun(@times, ySEM(p,q)', CI95); 
            yCI95 = yCI95';

            ax(p,q) = subplot(2, 5 ,counter);
            str = sprintf('Ch%d', q);
            title(str);
            % Plot Mean Of All Experiments
            plot(p, yMean(p,q), 'kx');
            hold on
            % Plot 95% Confidence Intervals Of All Experiments
            errorbar(p, yMean(p,q), yCI95(2), 'k'); 
            counter = counter + 1;
            grid
            set(gca, 'XLim', [0 4]);
            xticks([1 2 3]);
            xticklabels({'W00-ND' , 'W01-ND' , 'W01-BR'});
        end
        if s == 1
            counter = 1;
        else
            counter = 6;
        end
    end
end

function data_out = bufferData(data_in, N_max)
    buff = N_max - size(data_in, 1);
    if buff > 0
        data_out = [data_in; nan(buff, size(data_in,2))];
    else
        data_out = data_in;
    end
end
